% [phi_out] = gacs(seg_mask_in, g, arhs, params)
%                                      
% Matlab interface to a gacs evolver utilizing implicit time integration
% schemes and multigrid evolvers for simulating the model:              
%                            
%    du(x,y,t)/dt = |grad(u)| { div(g(x,y) (grad(u)/|grad(u)|)) + arhs },
%
% with adiabatic (zero Neumann) boundary conditions at the image boundaries.
%          
% 1. The initial condition is determined as follows: The user supplies a binary
% segmentation mask, which the program converts to u(x,y,0) via a signed       
% distance transformation.
%
% 2. The edge-map g(x,y) determines the riemannian geometry on the image plane
% and induces the internal forces to the contour; it is typically small near
% image edges; in the special case that g=1 everywhere, one obtains curve
% motion with speed proportional to curvature. The presense of the numerically
% stiff geometric heat term div(g(x,y) (grad(u)/|grad(u)|)) is that makes the
% problem difficult to solve with explicit in time schemes and demonstrates the
% advantages of implicit schemes/multigrid solvers for rapidly advancing the curve.
%
% 3. The arhs is a generic rhs term gathering all external forces, and is
% integrated in time by explicit schemes. By properly setting it one can easily
% implement various geometric active contour segmentation models; for example
% arhs can include Cohen's balloon force terms; Mumford-Shah's, Chan/Vese's,
% or Tsai et al.'s piecewise smooth image segmentation models; Zhu and Yuille's
% region competition models; and Leventon et al.'s or Rousson and Paragios'
% shape prior-based models.
%
% NOTE: the same arhs will be used for all iterations; if the arhs for the problem
% at hand depends on u (e.g. Chan-Vese's model), you must allow gacs evolve for
% only few iterations, then re-compute arhs, and call gacs again.
%
% Matlab call:
%
% [phi_out] = gacs(seg_mask_in, g, arhs, params);
%
% with:
%     phi_out       : double image / the signed distance transform of final segmentation (<0 inside)
%     seg_mask_in   : binary image / initial segmentation; true values in the interior of the domain
%     g (opt.)      : real image / edge indicator function [default: 1 everywhere]
%     arhs (opt.)   : real image / external forces [default: 0]
%     params (opt.) : parameter structure for the problem, the implicit evolver, and the multigrid solver
%                     * params.problem_conv_dist : convergence distance threshold
%                     *               _balloon : balloon force (>0 for erosion)
%                     * params.evolver_dt     : size of time-step
% 		      *               _steps  : max number of timesteps
% 		      * params.solver_levels  : number of multigrid levels
% 		      *              _eps     : residual eps - determines number of multigrid cycles
% 		      *              _output  : (0-5) determines amount of output
%
% For further details, check-out the paper
% G. Papandreou and P. Maragos, 'Multigrid Geometric Active Contour Models', IEEE Tr. on Image Proc.,
% available from http://cvsp.cs.ntua.gr/gpapan
%
% George Papandreou, January 2007
%

error('gacs is implemented as MEX dynamic library; add it to the Matlab path!');